@@ -26,7 +26,7 @@ lazy_static! {
         ("--- /dev/null", FAINT),
         ("+++ /dev/null", FAINT),
     ];
-    static ref ANSI_COLOR_REGEX: Regex = Regex::new("\x1b[^m]*m").unwrap();
+    static ref ANSI_COLOR_REGEX: Regex = Regex::new("\x1b\\[[0-9;]*[^0-9;]").unwrap();
 }
 
 #[must_use]
@@ -138,6 +138,7 @@ pub struct LineCollector {
     old_text: String,
     new_text: String,
     plain_text: String,
+    diff_seen: bool,
     consumer_thread: Option<JoinHandle<()>>,
 
     diffing_threads: ThreadPool,
@@ -208,6 +209,8 @@ impl LineCollector {
             old_text: String::from(""),
             new_text: String::from(""),
             plain_text: String::from(""),
+            diff_seen: false,
+
             consumer_thread: Some(consumer),
             diffing_threads: ThreadPool::new(num_cpus::get()),
             queue_putter,
@@ -325,10 +328,18 @@ impl LineCollector {
         self.consume_plain_line(NORMAL);
     }
 
+    fn without_ansi_escape_codes(input: &'_ str) -> std::borrow::Cow<'_, str> {
+        return ANSI_COLOR_REGEX.replace_all(input, "");
+    }
+
     pub fn consume_line(&mut self, line: String) {
         // Strip out incoming ANSI formatting. This enables us to highlight
         // already-colored input.
-        let line = ANSI_COLOR_REGEX.replace_all(&line, "");
+        let line = LineCollector::without_ansi_escape_codes(&line);
+
+        if line.starts_with("diff") {
+            self.diff_seen = true;
+        }
 
         if let Some(fixed_highlight) = get_fixed_highlight(&line) {
             self.consume_plain_linepart(fixed_highlight);
@@ -338,7 +349,7 @@ impl LineCollector {
         }
 
         if line.starts_with("commit") {
-            self.consume_plain_line(&format_commit_line(&line));
+            self.consume_plain_line(&format_commit_line(&line, self.diff_seen));
             return;
         }
 
@@ -375,3 +386,35 @@ impl LineCollector {
         self.consume_plain_line(&line);
     }
 }
+
+#[cfg(test)]
+mod tests {
+    use super::*;
+
+    #[cfg(test)]
+    use pretty_assertions::assert_eq;
+
+    #[test]
+    fn test_non_sgr() {
+        assert_eq!(
+            LineCollector::without_ansi_escape_codes("hel\x1b[0Klo"),
+            "hello"
+        );
+    }
+
+    #[test]
+    fn test_sgr() {
+        assert_eq!(
+            LineCollector::without_ansi_escape_codes("hel\x1b[33mlo"),
+            "hello"
+        );
+    }
+
+    #[test]
+    fn test_multi_sgr() {
+        assert_eq!(
+            LineCollector::without_ansi_escape_codes("hel\x1b[33;34mlo"),
+            "hello"
+        );
+    }
+}
